import 'dart:async';
import 'dart:convert';
import 'dart:io';

import 'package:flutter/cupertino.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter/widgets.dart';

import 'BusinessDocuments.dart';
import 'Constants.dart';
import 'Requests/BusinessDetailParams.dart';
import 'Requests/BusinessIdParams.dart';
import 'Responses/BanksResponse.dart';
import 'RetrofitConfigForee.dart';
import 'Utilities/Utils.dart';

class AccountDetailsForm extends StatefulWidget {
  final String apiKey;
  final UserDetails userDetails;
  final List<Bank> banks;
  final logo;
  final BusinessDetailParams businessDetails;
  final ValueSetter<String> callbackBusinessId;
  AccountDetailsForm({Key key, @required this.apiKey, this.banks,
    this.userDetails, this.logo, this.businessDetails,
    this.callbackBusinessId}) : super(key: key);

  @override
  _AccountDetailsState createState() => _AccountDetailsState();
}
class _AccountDetailsState extends State<AccountDetailsForm> {
  final _formKey = GlobalKey<FormState>();
  final _scaffoldStateKey = GlobalKey<ScaffoldState>();
  final title = Titles.BANK_ACCOUNT;

  AutovalidateMode _autoValidateMode;

  @override
  void initState() {
    _autoValidateMode = AutovalidateMode.disabled;
    if (widget.businessDetails.bankAccountDetails == null)
      widget.businessDetails.bankAccountDetails = BankAccountDetails();
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return WillPopScope(
        child: Scaffold(
            key: _scaffoldStateKey,
            appBar: AppBar(
              title: Text(title),
              leading: widget.logo,
              automaticallyImplyLeading: Platform.isIOS,
            ),
            body: Padding(
              padding: PaddingDetails.FORM,
              child: Column(
                mainAxisAlignment: MainAxisAlignment.spaceBetween,
                children: <Widget>[
                  Expanded(
                    child: ListView(
                      children: [
                        Form(
                          key: _formKey,
                          autovalidateMode: _autoValidateMode,
                          child: Column(
                            crossAxisAlignment: CrossAxisAlignment.start,
                            children: [
                              // Subtitle
                              Padding(
                                padding: PaddingDetails.FIELD_TOP,
                                child: Text(
                                  SubTitle.BANK_ACCOUNT_DESCRIPTION,
                                  style: TextStyling.getHeaderStyle(),
                                ),
                              ),
                              // Account title
                              Padding(
                                padding: PaddingDetails.FIELD_TOP,
                                child: TextFormField(
                                  decoration: InputDecoration(
                                      border: const OutlineInputBorder(),
                                      labelText: FormLabels.ACCOUNT_TITLE
                                  ),
                                  validator: (value) {
                                    if (value.isEmpty) {
                                      return Utils.showEnterFieldError("account title");
                                    } else if (value.length < LengthLimit.MIN_LENGTH_5) {
                                      return ErrorFields.MIN_LENGTH_ERROR_5;
                                    } else if (value.length > LengthLimit.MAX_LENGTH_100) {
                                      return ErrorFields.MAX_LENGTH_ERROR_100;
                                    } else return null;
                                  },
                                  onChanged: (value) => widget.businessDetails.bankAccountDetails.accountTitle = value.trim(),
                                  textCapitalization: TextCapitalization.sentences,
                                  initialValue: widget.businessDetails.bankAccountDetails.accountTitle,
                                ),
                              ),
                              // IBan
                              Padding(
                                padding: PaddingDetails.FIELD_TOP,
                                child: TextFormField(
                                  decoration: InputDecoration(
                                      border: const OutlineInputBorder(),
                                      labelText: FormLabels.IBAN
                                  ),
                                  validator: (value) {
                                    if (value.isEmpty) {
                                      return Utils.showEnterFieldError("IBAN");
                                    } else return null;
                                  },
                                  onChanged: (value) => widget.businessDetails.bankAccountDetails.iBan = value.trim(),
                                  keyboardType: TextInputType.visiblePassword,
                                  inputFormatters: [
                                    new LengthLimitingTextInputFormatter(24),
                                    new FilteringTextInputFormatter.allow(RegExp('[a-zA-Z0-9]')),
                                  ],
                                  initialValue: widget.businessDetails.bankAccountDetails.iBan,
                                ),
                              ),
                              // Account Number
                              Padding(
                                padding: PaddingDetails.FIELD_TOP,
                                child: TextFormField(
                                  decoration: InputDecoration(
                                      border: const OutlineInputBorder(),
                                      labelText: FormLabels.ACCOUNT_NUMBER
                                  ),
                                  validator: (value) {
                                    if (value.isEmpty) {
                                      return Utils.showEnterFieldError("account number");
                                    } else if (value.length < LengthLimit.MIN_LENGTH_1) {
                                      return ErrorFields.MIN_LENGTH_ERROR_1;
                                    } else return null;
                                  },
                                  onChanged: (value) => widget.businessDetails.bankAccountDetails.accountNumber = value,
                                  keyboardType: TextInputType.phone,
                                  inputFormatters: [
                                    new LengthLimitingTextInputFormatter(20),
                                    new FilteringTextInputFormatter.allow(RegExp("[0-9]"))
                                  ],
                                  initialValue: widget.businessDetails.bankAccountDetails.accountNumber,
                                ),
                              ),
                              // Bank Name
                              Padding(
                                padding: PaddingDetails.FIELD_TOP,
                                child: DropdownButtonFormField(
                                  decoration: InputDecoration(
                                      border: OutlineInputBorder(),
                                      labelText: FormLabels.BANK_NAME
                                  ),
                                  onTap: () => FocusScope.of(context)
                                      .requestFocus(FocusNode()),
                                  isExpanded: true,
                                  value: widget.businessDetails.bankAccountDetails.bankId,
                                  items: Utils.banks.map((bank) =>
                                      DropdownMenuItem(
                                        child: Text(bank.name),
                                        value: bank.bankId,
                                      )
                                  ).toList(),
                                  onChanged: (value) {
                                    setState(() {
                                      widget.businessDetails.bankAccountDetails.bankId = value;
                                    });
                                  },
                                  validator: (value) {
                                    if (value == null)
                                      return Utils.showEnterFieldError("bank name");
                                    else return null;
                                  },
                                ),
                              ),
                              // Branch Name
                              Padding(
                                padding: PaddingDetails.FIELD_TOP,
                                child: TextFormField(
                                  decoration: InputDecoration(
                                      border: const OutlineInputBorder(),
                                      labelText: FormLabels.BRANCH_NAME
                                  ),
                                  validator: (value) {
                                    if (value.isEmpty) {
                                      return Utils.showEnterFieldError("branch name");
                                    } else return null;
                                  },
                                  onChanged: (value) => widget.businessDetails.bankAccountDetails.branchName = value.trim(),
                                  textCapitalization: TextCapitalization.sentences,
                                  initialValue: widget.businessDetails.bankAccountDetails.branchName,
                                ),
                              ),
                              // Branch Code
                              Padding(
                                padding: PaddingDetails.FIELD_TOP,
                                child: TextFormField(
                                  decoration: InputDecoration(
                                      border: const OutlineInputBorder(),
                                      labelText: FormLabels.BRANCH_CODE
                                  ),
                                  validator: (value) {
                                    if (value.isEmpty) {
                                      return Utils.showEnterFieldError("branch code");
                                    } else return null;
                                  },
                                  onChanged: (value) => widget.businessDetails.bankAccountDetails.branchCode = value,
                                  initialValue: widget.businessDetails.bankAccountDetails.branchCode,
                                  keyboardType: TextInputType.phone,
                                  inputFormatters: [
                                    LengthLimitingTextInputFormatter(5),
                                    FilteringTextInputFormatter.allow(RegExp("[0-9]"))
                                  ],
                                ),
                              ),
                            ],
                          ),
                        ),
                      ],
                    )
                  ),
                  // Buttons
                  Padding(
                    padding: PaddingDetails.NOTE,
                    child: Column(
                      crossAxisAlignment: CrossAxisAlignment.end,
                      children: [
                        Row(
                          mainAxisAlignment: MainAxisAlignment.spaceBetween,
                          children: [
                            Utils.getHelplineMark(),
                            Utils.getLogoMark(),
                          ],
                        ),
                        Row(
                          children: [
                            // Button Save
                            Expanded(
                              flex: 2,
                              child: Padding(
                                padding: PaddingDetails.BTN_RIGHT,
                                child: SizedBox(
                                  width: Layout.BTN_WIDTH,
                                  child: ElevatedButton(
                                    onPressed: () {
                                      if (widget.businessDetails.bankAccountDetails.bankId == null) {
                                        Utils.showSnackBar(_scaffoldStateKey.currentState, "Please select Bank name first");
                                        return;
                                      }
                                      onBoardBusiness(
                                          isSubmitting: false,
                                          moveToNext: false
                                      );
                                    },
                                    child: Text(
                                      ButtonTags.SAVE,
                                      style: TextStyle(
                                        color: Theme.of(context).primaryColor,
                                      ),
                                    ),
                                    style: ButtonStyle(
                                        backgroundColor: MaterialStateColor.resolveWith(
                                                (states) => Colors.white
                                        )
                                    ),
                                  ),
                                ),
                              ),
                            ),
                            // Button next
                            Expanded(
                              flex: 2,
                              child: Padding(
                                padding: PaddingDetails.BTN_LEFT,
                                child: SizedBox(
                                  width: Layout.BTN_WIDTH,
                                  child: ElevatedButton(
                                      onPressed: () async {
                                        if (_formKey.currentState.validate()) {
                                          onBoardBusiness(
                                              isSubmitting: false,
                                              moveToNext: true
                                          );
                                        } else {
                                          setState(() {
                                            _autoValidateMode = AutovalidateMode.always;
                                          });
                                        }
                                      },
                                      child: Text(
                                        ButtonTags.NEXT,
                                      )
                                  ),
                                ),
                              ),
                            )
                          ],
                        )
                      ],
                    ),
                  )
                ],
              ),
            )
        ),
        onWillPop: () {
          Navigator.of(context).pop(false);
          return Future.value(false);
        }
    );
  }
  void businessDocumentsFlow(BusinessDetailParams businessDetails) async {
    Utils.fetchDocuments(widget.apiKey, BusinessIdParams(
        businessId: businessDetails.businessId
    )).then((value) {
      var data = json.decode(value);
      if (data != null && data["response_code"] == 0) {
        Navigator.of(context).pop(false);
        Map<String, dynamic> docs = data["response_data"]["documents"];
        Navigator.push(context, MaterialPageRoute(
            builder: (context) => BusinessDocuments(
              apiKey: widget.apiKey,
              userDetails: UserDetails(),
              businessDetails: businessDetails,
              documents: docs,
              callbackBusinessId: widget.callbackBusinessId,
            )
        )).then((value) {
          if (value != null && value[0] && value[1])
            Navigator.of(context).pop(value);
        });
      } else {
        Navigator.of(context).pop(false);
        Utils.showSnackBar(_scaffoldStateKey.currentState, data["response_message"]);
      }
    }).timeout(
        Duration(seconds: Utils.timeOutSec),
        onTimeout: () { Navigator.of(context).pop(false); return; }
    ).catchError((onError) {
      print(onError);
      Navigator.of(context).pop(false);
    });
  }
  void onBoardBusiness({bool isSubmitting, bool moveToNext}) async {
    widget.businessDetails.isSubmitting = isSubmitting;
    Utils.apiCallsHandler(context, _scaffoldStateKey.currentState, () async {
      moveToNext ? Utils.showLoaderDialogWithText(context, "Please wait...")
          : Utils.showLoaderDialogWithText(context, "Saving...");
      await Utils.onBoardBusiness(widget.apiKey, widget.businessDetails).then((response) {
        if (response != null) {
          var data = json.decode(response);
          var responseCode = data['response_code'];
          var responseMessage = data['response_message'];
          if (responseCode == 0) {
            if (moveToNext)
              businessDocumentsFlow(widget.businessDetails);
            else {
              widget.callbackBusinessId(widget.businessDetails.businessId);
              Navigator.of(context).pop(false);
              Utils.showSnackBar(_scaffoldStateKey.currentState, General.showSaveResponse2());
            }
          } else {
            Navigator.of(context).pop(false);
            Utils.showSnackBar(_scaffoldStateKey.currentState, responseMessage);
          }
        }
      }).timeout(
          Duration(seconds: Utils.timeOutSec),
          onTimeout: () {
            Navigator.of(context).pop(false);
            Utils.showSnackBar(_scaffoldStateKey.currentState, GeneralError.SERVICE_UNAVAILABLE);
          }
      ).catchError((onError) {
        print(onError);
        Navigator.of(context).pop(false);
        Utils.showNetworkErrors(_scaffoldStateKey.currentState, onError);
      });
    });
  }
}