import 'dart:convert';
import 'dart:io';

import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';

import 'BusinessDetails.dart';
import 'Constants.dart';
import 'Requests/BusinessDetailParams.dart';
import 'Requests/BusinessIdParams.dart';
import 'Requests/ForgotPasswordParams.dart';
import 'Requests/LoginOrCreatePasswordParams.dart';
import 'Responses/User.dart';
import 'RetrofitConfigForee.dart';
import 'UserBusinesses.dart';
import 'Utilities/Utils.dart';

class PasswordScreen extends StatefulWidget {
  final String apiKey;
  final UserDetails userDetails;
  final Map<String, String> data;
  final bool isLogin;
  final User user;
  final BusinessDetailParams businessDetails;
  final ValueSetter<String> callbackBusinessId;
  PasswordScreen({Key key, @required this.apiKey, this.data, this.userDetails,
    this.isLogin, this.user, this.businessDetails, this.callbackBusinessId})
      : super(key: key);
  @override
  _PasswordScreenState createState() => _PasswordScreenState();
}

class _PasswordScreenState extends State<PasswordScreen> {
  final _formKey = GlobalKey<FormState>();
  final _scaffoldStateKey = GlobalKey<ScaffoldState>();

  Widget logo;
  Icon _passwordIcon = Icon(Icons.visibility_off);
  bool _obscurePassword = true;
  String _password;
  AutovalidateMode _autoValidateMode;
  RegExp regex = new RegExp(r'^(?=.*?[A-Z])(?=.*?[a-z])(?=.*?[0-9])(?=.*?[!@#\$&*~]).{8,}$');

  @override
  void initState() {
    _autoValidateMode = AutovalidateMode.disabled;
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return WillPopScope(
        child: Scaffold(
            key: _scaffoldStateKey,
            appBar: AppBar(
              title: Text(
                  widget.isLogin ? Titles.ENTER_PASSWORD : Titles.CREATE_PASSWORD
              ),
              automaticallyImplyLeading: Platform.isIOS,
              leading: logo,
            ),
            body: Padding(
              padding: PaddingDetails.FORM,
              child: Form(
                key: _formKey,
                autovalidateMode: _autoValidateMode,
                child: Column(
                  mainAxisAlignment: MainAxisAlignment.spaceBetween,
                  children: <Widget>[
                    Expanded(
                      child: Column(
                        crossAxisAlignment: CrossAxisAlignment.start,
                        children: [
                          // Password
                          Padding(
                            padding: PaddingDetails.FIELD_TOP,
                            child: TextFormField(
                              decoration: InputDecoration(
                                  border: const OutlineInputBorder(),
                                  labelText: FormLabels.PASSWORD,
                                  suffixIcon: IconButton(
                                      icon: _passwordIcon,
                                      color: Colors.black,
                                      onPressed: () => _showHidePassword()
                                  )
                              ),
                              validator: (value) {
                                if (value.isEmpty || !regex.hasMatch(value))
                                  return Utils.showEnterFieldError("valid password");
                                else if (value.length < LengthLimit.MIN_LENGTH_8)
                                  return ErrorFields.MIN_LENGTH_ERROR_8;
                                else return null;
                              },
                              onChanged: (value) => _password = value.trim(),
                              keyboardType: TextInputType.visiblePassword,
                              obscureText: _obscurePassword,
                              enableSuggestions: false,
                            ),
                          ),
                          // Password hint
                          Visibility(
                              visible: !widget.isLogin,
                              child: Padding(
                                padding: PaddingDetails.HINT,
                                child: Text(General.PASSWORD_HINT),
                              )
                          ),
                          // Forgot password
                          Visibility(
                            visible: widget.isLogin,
                            child: Padding(
                              padding: PaddingDetails.HINT,
                              child: InkWell(
                                child: Text(
                                  TextOf.FORGOT_PASS,
                                  style: TextStyle(
                                      fontWeight: FontWeight.bold,
                                      color: Theme.of(context).primaryColor
                                  ),
                                ),
                                onTap: () {
                                  Utils.apiCallsHandler(context, _scaffoldStateKey
                                      .currentState, () {
                                    Utils.showLoaderDialogWithText(context, "Please wait...");
                                    Utils.forgotPassword(widget.apiKey,
                                        ForgotPasswordParams(
                                            userName: widget.user.responseData.userName
                                        )
                                    ).then((value) {
                                      Navigator.of(context).pop(false);
                                      Utils.showSnackBar(_scaffoldStateKey.currentState, value.responseMessage);
                                    }).timeout(
                                        Duration(seconds: 30),
                                        onTimeout: () { Navigator.of(context).pop(false); }
                                    ).catchError((onError) {
                                      print(onError);
                                      Navigator.of(context).pop(false);
                                    });
                                  });
                                },
                              ),
                            ),
                          )
                        ],
                      ),
                    ),
                    // Next button
                    Container(
                      width: Layout.BTN_WIDTH,
                      child: Column(
                        crossAxisAlignment: CrossAxisAlignment.end,
                        children: [
                          Row(
                            mainAxisAlignment: MainAxisAlignment.spaceBetween,
                            children: [
                              Utils.getHelplineMark(),
                              Utils.getLogoMark(),
                            ],
                          ),
                          SizedBox(
                            width: Layout.BTN_WIDTH,
                            child: ElevatedButton(
                                onPressed: () async {
                                  if (_formKey.currentState.validate()) {
                                    Utils.apiCallsHandler(context, _scaffoldStateKey
                                        .currentState, () {
                                      Utils.showLoaderDialogWithText(context, "Please wait...");
                                      if (widget.isLogin) {
                                        _loginFlow(LoginOrCreatePasswordParams(
                                            userId: widget.user.responseData.id,
                                            password: _password
                                        ));
                                      } else {
                                        _createPasswordFlow(
                                            LoginOrCreatePasswordParams(
                                                userId: widget.user.responseData.id,
                                                password: _password
                                            )
                                        );
                                      }
                                    });
                                  } else {
                                    setState(() {
                                      _autoValidateMode = AutovalidateMode.always;
                                    });
                                  }
                                },
                                child: Text(
                                    ButtonTags.NEXT
                                )
                            ),
                          )
                        ],
                      ),
                    )
                  ],
                ),
              ),
            )
        ),
        onWillPop: () {
          Navigator.of(context).pop([true, false]);
          return Future.value(false);
        }
    );
  }
  void _showHidePassword() {
    setState(() {
      _obscurePassword = !_obscurePassword;
      if (_obscurePassword) _passwordIcon = Icon(Icons.visibility_off);
      else _passwordIcon = Icon(Icons.visibility);
    });
  }
  void _createPasswordFlow(LoginOrCreatePasswordParams params) async {
    await Utils.createPassword(widget.apiKey, params).then((user) {
      if (user != null) {
        if (user.responseCode == 0) {
          Navigator.of(context).pop(false);
          if (user.responseData.businessLandingScreenRequired) {
            _redirectToUserBusinesses(user);
          } else {
            _redirectToBusinessDetails(
                user: user,
                businessDetails: widget.businessDetails
            );
          }
        }
      }
    }).timeout(
        Duration(seconds: Utils.timeOutSec),
        onTimeout: () {
          Navigator.of(context).pop(false);
          Utils.showSnackBar(_scaffoldStateKey.currentState, GeneralError.SERVICE_UNAVAILABLE);
        }
    ).catchError((onError) {
      print(onError);
      Navigator.of(context).pop(false);
      Utils.showNetworkErrors(_scaffoldStateKey.currentState, onError);
    });
  }
  void _loginFlow(LoginOrCreatePasswordParams params) async {
    Utils.login(widget.apiKey, params).then((user) {
      if (user != null) {
        if (user.responseCode == 0) {
          print("user ${user.responseData.toJson().toString()}");
          if (user.responseData.businessLandingScreenRequired) {
            Navigator.of(context).pop(false);
            _redirectToUserBusinesses(user);
          } else {
            Businesses business = user.responseData.businesses.isNotEmpty? user.responseData.businesses.first : null;
            if (business != null) {
              if(business.status.businessStatus == "Data Pending") {
                Utils.fetchBusinessDetails(widget.apiKey, BusinessIdParams(
                    businessId: user.responseData.businessIds.first
                )).then((response) {
                  if (response != null) {
                    var data = json.decode(response);
                    var responseCode = data['response_code'];
                    var responseMessage = data['response_message'];
                    if (responseCode == 0) {
                      var business = data['response_data']['business'];
                      Navigator.of(context).pop(false);
                      _redirectToBusinessDetails(
                          user: user,
                          businessDetails: BusinessDetailParams.fromJson(business)
                      );
                    } else {
                      Navigator.of(context).pop(false);
                      Utils.showSnackBar(_scaffoldStateKey.currentState, responseMessage);
                    }
                  }
                }).timeout(
                    Duration(seconds: Utils.timeOutSec),
                    onTimeout: () {
                      Navigator.of(context).pop(false);
                      Utils.showSnackBar(_scaffoldStateKey.currentState, GeneralError.SERVICE_UNAVAILABLE);
                    }
                ).catchError((onError) {
                  print(onError);
                  Navigator.of(context).pop(false);
                  Utils.showNetworkErrors(_scaffoldStateKey.currentState, onError);
                });
              }
            } else {
              _redirectToBusinessDetails(
                  user: user,
                  businessDetails: widget.businessDetails
              );
            }
          }
        }
        else {
          Navigator.of(context).pop(false);
          Utils.showSnackBar(_scaffoldStateKey.currentState, user.responseMessage);
        }
      }
    }).timeout(
        Duration(seconds: Utils.timeOutSec),
        onTimeout: () {
          Navigator.of(context).pop(false);
          Utils.showSnackBar(_scaffoldStateKey.currentState, GeneralError.SERVICE_UNAVAILABLE);
        }
    ).catchError((onError) {
      print(onError);
      Navigator.of(context).pop(false);
      Utils.showNetworkErrors(_scaffoldStateKey.currentState, onError);
    });;
  }
  void _redirectToBusinessDetails({User user, BusinessDetailParams businessDetails}) {
    Navigator.push(context, MaterialPageRoute(builder: (context) =>
        BusinessDetails(
          apiKey: widget.apiKey,
          data: widget.data,
          user: user,
          businessDetails: businessDetails,
          callbackBusinessId: widget.callbackBusinessId,
        )
    )).then((value) {
      print(value);
      if (value != null && value[0] && value[1])
        Navigator.of(context).pop(value);
      else Navigator.of(context).pop([true, false]);
    });
  }
  void _redirectToUserBusinesses(User user) {
    Navigator.pushReplacement(context, MaterialPageRoute(builder: (context) =>
        UserBusinesses(
          apiKey: widget.apiKey,
          userId: user.responseData.id,
          data: widget.data,
          userDetails: widget.userDetails,
          businessList: user.responseData.businessIds,
        )
    ));
  }
}