import 'package:businessonboarding/src/Utilities/Utils.dart';
import 'package:json_annotation/json_annotation.dart';
import 'package:retrofit/retrofit.dart';
import 'package:dio/dio.dart';

import 'Constants.dart';
import 'Requests/AcquireUserParams.dart';
import 'Requests/BusinessDetailParams.dart';
import 'Requests/BusinessIdParams.dart';
import 'Requests/ForgotPasswordParams.dart';
import 'Requests/LoginOrCreatePasswordParams.dart';
import 'Requests/OtpParams.dart';
import 'Requests/UpdateUserParams.dart';
import 'Requests/VerifyParams.dart';
import 'Responses/BanksResponse.dart';
import 'Responses/City.dart';
import 'Responses/FeeBorne.dart';
import 'Responses/ForgotPassword.dart';
import 'Responses/InstrumentType.dart';
import 'Responses/Otp.dart';
import 'Responses/Province.dart';
import 'Responses/User.dart';

part 'RetrofitConfigForee.g.dart';

abstract class ForeeRestClient {
  factory ForeeRestClient(Dio dio) = _ForeeRestClient;

  @POST("/sdk/on-boarding/version")
  Future<String> getVersion(@Body() VersionInfo versionInfo);

  @POST("/aggregator/business/signup")
  Future<String> signUp(@Body() UserDetails signUp);

  @GET("/static/static_business_legal_entity")
  Future<String> fetchLegalEntities();

  @GET("/static/static_business_category")
  Future<String> fetchBusinessCategories();

  @GET("/static/static_bank")
  Future<List<Bank>> fetchBanks();

  @GET("/static/static_office_premises")
  Future<String> fetchOfficePremises();

  @POST("/aggregator/business-details")
  Future<String> fetchUserDetails(@Body() UserDetailsRequest request);

  @GET("/static/static_cities")
  Future<List<City>> fetchCities();

  @GET("/static/static_provinces")
  Future<List<Province>> fetchProvinces();

  @GET("/static/static_fee_borne")
  Future<List<FeeBorne>> fetchFeeBorne();

  @GET("/static/static_instrument_types")
  Future<List<InstrumentType>> fetchInstrumentTypes();

  // Version 2 calls
  @POST("/user/acquire")
  Future<User> acquireUser(@Body() AcquireUser acquireUser);
  @POST("/user/update")
  Future<User> updateUser(@Body() UpdateUser updateUser);
  @POST("/user/create-password")
  Future<User> createPassword(@Body() LoginOrCreatePasswordParams params);
  @POST("/user/login")
  Future<User> login(@Body() LoginOrCreatePasswordParams params);
  @POST("/user/forgot_password")
  Future<ForgotPassword> forgotPassword(@Body() ForgotPasswordParams params);
  @POST("/user/resend-email")
  Future<Otp> sendOtpToEmail(@Body() OtpParams params);
  @POST("/user/resend-sms")
  Future<Otp> sendOtpToNumber(@Body() OtpParams params);
  @POST("/v2/user/verify-email")
  Future<User> verifyEmail(@Body() VerifyParams params);
  @POST("/v2/user/verify-phone")
  Future<User> verifyPhone(@Body() VerifyParams params);
  @POST("/business/onboard")
  Future<String> onBoardBusiness(@Body() BusinessDetailParams params);
  @POST("/v2/business-details")
  Future<String> fetchBusinessDetails(@Body() BusinessIdParams params);
  @POST("/v2/business/upload-docs")
  Future<String> uploadImageToServer(@Body() FormData formData);
  @POST("/v2/business/remove-docs")
  Future<String> removeImage(@Body() RemoveImage removeImage);
  @POST("/business/get-docs")
  Future<String> fetchDocuments(@Body() BusinessIdParams params);
}

@JsonSerializable()
class VersionInfo {
  String sdkPlatform;
  String sdkAppVersion;
  VersionInfo({ this.sdkPlatform, this.sdkAppVersion });
  factory VersionInfo.fromJson(Map<String, dynamic> json) =>
      _$VersionFromJson(json);
  Map<String, dynamic> toJson() => _$VersionToJson(this);
}

VersionInfo _$VersionFromJson(Map<String, dynamic> json) {
  return VersionInfo(
    sdkPlatform: json[JsonFieldKey.SDK_PLATFORM] as String,
    sdkAppVersion: json[JsonFieldKey.SDK_APP_VERSION] as String,
  );
}

Map<String, dynamic> _$VersionToJson(VersionInfo instance) =>
    <String, dynamic> {
  JsonFieldKey.SDK_PLATFORM: instance.sdkPlatform,
  JsonFieldKey.SDK_APP_VERSION: instance.sdkAppVersion,
};

@JsonSerializable()
class UserDetails {
  String emailAddress;
  String mobileNumber;
  String password;
  String businessLegalName;
  String businessName;
  String brandShopName;
  String businessDescription;
  String registeredNumber;
  String registeredAddress;
  String physicalAddress;
  BusinessType businessType;
  String ntnCnic;
  String nameOfBusinessOwner;
  String cnicOfBusinessOwner;
  BusinessCategories businessCategories;

  OfficePremises officePremises;
  String yearAtCurrentLocation;
  String yearAtCurrentBusiness;
  String expectedMonthlyTurnOver;

  BankAccountDetails accountDetails;

  String officialWebsite;
  String businessId;

  bool isSubmitting;
  String logoUrl;
  Documents documents;
  List<dynamic> partnerDetails;

  UserDetails({this.nameOfBusinessOwner, this.cnicOfBusinessOwner,
    this.businessId, this.businessLegalName, this.businessCategories,
    this.emailAddress, this.expectedMonthlyTurnOver, this.businessType,
    this.mobileNumber, this.ntnCnic, this.officialWebsite,
    this.officePremises, this.physicalAddress, this.password,
    this.registeredNumber, this.registeredAddress, this.yearAtCurrentLocation,
    this.yearAtCurrentBusiness, this.accountDetails, this.isSubmitting,
    this.logoUrl, this.documents, this.businessName, this.partnerDetails, this.brandShopName});
  factory UserDetails.fromJson(Map<String, dynamic> json) => _$UserDetailsFromJson(json);
  Map<String, dynamic> toJson() => _$UserDetailsToJson(this);
}

UserDetails _$UserDetailsFromJson(Map<String, dynamic> json) {
  final categories = json[JsonFieldKey.BUSINESS_CATEGORY];
  final entities = json[JsonFieldKey.BUSINESS_LEGAL_ENTITY];
  final premises = json[JsonFieldKey.OFFICE_PREMISES];
  final rawList = json['linked_banks'] as List;
  final documents = json[JsonFieldKey.DOCUMENTS];
  return UserDetails(
    businessId: json[JsonFieldKey.ID] as String,
    nameOfBusinessOwner: json[JsonFieldKey.AUTHORIZED_SIGNATORY_NAME] as String,
    cnicOfBusinessOwner: json[JsonFieldKey.AUTHORIZED_SIGNATORY_CNIC] as String,
    businessLegalName: json[JsonFieldKey.BUSINESS_NAME] as String,
    businessCategories: (categories != null ? BusinessCategories.fromJson(categories) : BusinessCategories()),
    emailAddress: json[JsonFieldKey.EMAIL_ADDRESS] as String,
    expectedMonthlyTurnOver: json[JsonFieldKey.EXPECTED_MONTHLY_TURNOVER] as String,
    businessType: (entities != null ? BusinessType.fromJson(entities) : BusinessType()),
    mobileNumber: json[JsonFieldKey.MOBILE_NUMBER] as String,
    ntnCnic: json[JsonFieldKey.NTN] as String,
    officialWebsite: json[JsonFieldKey.WEBSITE_URL] as String,
    officePremises: (premises != null ? OfficePremises.fromJson(premises) : OfficePremises()),
    physicalAddress: json[JsonFieldKey.PHYSICAL_ADDRESS] as String,
    password: json[JsonFieldKey.PASSWORD] as String,
    registeredNumber: json[JsonFieldKey.LAND_LINE_NUMBER] as String,
    registeredAddress: json[JsonFieldKey.ADDRESS] as String,
    yearAtCurrentLocation: json[JsonFieldKey.YEARS_AT_CURRENT_LOCATION] as String,
    yearAtCurrentBusiness: json[JsonFieldKey.YEARS_IN_BUSINESS] as String,
    accountDetails: ((rawList != null && rawList.isNotEmpty) ? BankAccountDetails.fromJson(rawList[0]) : BankAccountDetails()),
    isSubmitting: json[JsonFieldKey.FORM_SUBMITTED] as bool,
    logoUrl: json[JsonFieldKey.BUSINESS_LOGO] as String,
    documents: (documents != null ? Documents.fromJson(documents) : Documents()),
    businessName: json[JsonFieldKey.DOING_BUSINESS_AS] as String,
    partnerDetails: json[JsonFieldKey.PARTNERS_DETAILS] as List,
    brandShopName: json[JsonFieldKey.BUSINESS_SHORT_NAME] as String
  );
}

Map<String, dynamic> _$UserDetailsToJson(UserDetails instance) => <String, dynamic> {
  JsonFieldKey.AUTHORIZED_SIGNATORY_NAME: instance.nameOfBusinessOwner,
  JsonFieldKey.AUTHORIZED_SIGNATORY_CNIC: instance.cnicOfBusinessOwner,
  JsonFieldKey.BUSINESS_ID: instance.businessId,
  JsonFieldKey.BUSINESS_LEGAL_NAME: instance.businessLegalName,
  JsonFieldKey.BUSINESS_NAME: instance.businessLegalName,
  JsonFieldKey.BUSINESS_CATEGORY: (instance.businessCategories == null ? new Map<String, dynamic>() : instance.businessCategories.toJson()),
  JsonFieldKey.EMAIL_ADDRESS: instance.emailAddress,
  JsonFieldKey.EXPECTED_MONTHLY_TURNOVER: instance.expectedMonthlyTurnOver,
  JsonFieldKey.BUSINESS_LEGAL_ENTITY: (instance.businessType == null ? new Map<String, dynamic>() : instance.businessType.toJson()),
  JsonFieldKey.MOBILE_NUMBER: instance.mobileNumber,
  JsonFieldKey.NTN: instance.ntnCnic,
  JsonFieldKey.WEBSITE_URL: instance.officialWebsite,
  JsonFieldKey.OFFICE_PREMISES: (instance.officePremises == null ? new Map<String, dynamic>() : instance.officePremises.toJson()),
  JsonFieldKey.PHYSICAL_ADDRESS: instance.physicalAddress,
  JsonFieldKey.PASSWORD: instance.password,
  JsonFieldKey.LAND_LINE_NUMBER: instance.registeredNumber,
  JsonFieldKey.ADDRESS: instance.registeredAddress,
  JsonFieldKey.YEARS_AT_CURRENT_LOCATION: instance.yearAtCurrentLocation,
  JsonFieldKey.YEARS_IN_BUSINESS: instance.yearAtCurrentBusiness,
  JsonFieldKey.LINKED_BANKS: (instance.accountDetails == null ? new Map<String, dynamic>() : instance.accountDetails.toJson()),
  JsonFieldKey.FORM_SUBMITTED: instance.isSubmitting,
  JsonFieldKey.DOING_BUSINESS_AS: instance.businessName,
  JsonFieldKey.PARTNERS_DETAILS: (instance.partnerDetails == null ? new List() : instance.partnerDetails),
  JsonFieldKey.BUSINESS_SHORT_NAME: instance.brandShopName
};

@JsonSerializable()
class BusinessCategories {
  int id;
  String name;
  BusinessCategories({ this.id, this.name });
  factory BusinessCategories.fromJson(Map<String, dynamic> json) =>
      _$BusinessCategoriesFromJson(json);
  Map<String, dynamic> toJson() => _$BusinessCategoriesToJson(this);
}

BusinessCategories _$BusinessCategoriesFromJson(Map<String, dynamic> json) {
  return BusinessCategories(
      id: (json == null ? 0 : json[JsonFieldKey.ID] as int),
      name: (json == null ? "" : json[JsonFieldKey.NAME] as String)
  );
}

Map<String, dynamic> _$BusinessCategoriesToJson(BusinessCategories instance) =>
    <String, dynamic> {
  JsonFieldKey.ID: instance.id,
  JsonFieldKey.NAME: instance.name
};

@JsonSerializable()
class SettlementPeriod {
  String payment_channel;
  int settlement_days;
  SettlementPeriod({this.payment_channel, this.settlement_days});
  factory SettlementPeriod.fromJson(Map<String, dynamic> json) =>
      _$SettlementPeriodFromJson(json);
  Map<String, dynamic> toJson() => _$SettlementPeriodToJson(this);
}
SettlementPeriod _$SettlementPeriodFromJson(Map<String, dynamic> json) {
  return SettlementPeriod(
      payment_channel: (json == null ? "" : json['payment_channel'] as String),
      settlement_days: (json == null ? 0 : json['settlement_days'] as int)
  );
}
Map<String, dynamic> _$SettlementPeriodToJson(SettlementPeriod instance) =>
    <String, dynamic> {
      "payment_channel": instance.payment_channel,
      "settlement_days": instance.settlement_days
    };

@JsonSerializable()
class BusinessType {
  int id;
  String name;
  BusinessType({ this.id, this.name });
  factory BusinessType.fromJson(Map<String, dynamic> json) =>
      _$LegalEntitiesFromJson(json);
  Map<String, dynamic> toJson() => _$LegalEntitiesToJson(this);
}

BusinessType _$LegalEntitiesFromJson(Map<String, dynamic> json) {
  return BusinessType(
      id: json[JsonFieldKey.ID] as int,
      name: json[JsonFieldKey.NAME] as String
  );
}

Map<String, dynamic> _$LegalEntitiesToJson(BusinessType instance) =>
    <String, dynamic> {
  JsonFieldKey.ID: (instance == null ? 0 : instance.id),
  JsonFieldKey.NAME: (instance == null ? "" : instance.name)
};

@JsonSerializable()
class BankAccountDetails {
  // String id;
  String accountTitle;
  String iBan;
  String accountNumber;
  String bankId;
  String bankName;
  String branchName;
  String branchCode;
  BankAccountDetails({ this.accountTitle, this.accountNumber, this.branchName,
    this.branchCode, this.iBan, this.bankId, this.bankName });
  factory BankAccountDetails.fromJson(Map<String, dynamic> json) => _$BankAccountDetailsFromJson(json);
  Map<String, dynamic> toJson() => _$BankAccountDetailsToJson(this);
}

BankAccountDetails _$BankAccountDetailsFromJson(Map<String, dynamic> json) {
  print(json);
  var b = BankAccountDetails(
    accountTitle: json[JsonFieldKey.ACCOUNT_TITLE] as String,
    accountNumber: json[JsonFieldKey.ACCOUNT_NUMBER] as String,
    bankId: json[JsonFieldKey.BANK_ID] as String,
    branchName: json[JsonFieldKey.BRANCH_NAME] as String,
    branchCode: json[JsonFieldKey.BRANCH_CODE] as String,
    iBan: json[JsonFieldKey.IBAN] as String,
    bankName: json[JsonFieldKey.BANK_NAME] as String,
    // id: json[JsonFieldKey.ID] as String
  );
  return b;
}

Map<String, dynamic> _$BankAccountDetailsToJson(BankAccountDetails instance) =>
    <String, dynamic> {
  JsonFieldKey.ACCOUNT_TITLE: instance.accountTitle,
  JsonFieldKey.ACCOUNT_NUMBER: instance.accountNumber,
  JsonFieldKey.BANK_NAME: instance.bankName,
  JsonFieldKey.BRANCH_NAME: instance.branchName,
  JsonFieldKey.BRANCH_CODE: instance.branchCode,
  JsonFieldKey.IBAN: instance.iBan,
  JsonFieldKey.BANK_ID: instance.bankId,
  // JsonFieldKey.ID: instance.id
};

@JsonSerializable()
class PartnersDetails {
  String partnerName, partnerCnic;
  PartnersDetails({ this.partnerName, this.partnerCnic });
  factory PartnersDetails.fromJson(Map<String, dynamic> json) => _$PartnersDetailsFromJson(json);
  Map<String, dynamic> toJson() => _$PartnersDetailsToJson(this);
}

PartnersDetails _$PartnersDetailsFromJson(Map<String, dynamic> json) {
  return PartnersDetails(
      partnerName: json[JsonFieldKey.PARTNER_NAME] as String,
      partnerCnic: json[JsonFieldKey.PARTNER_CNIC] as String
  );
}

Map<String, dynamic> _$PartnersDetailsToJson(PartnersDetails instance) =>
    <String, dynamic> {
  JsonFieldKey.PARTNER_NAME: instance.partnerName,
  JsonFieldKey.PARTNER_CNIC: instance.partnerCnic
};

@JsonSerializable()
class OfficePremises {
  int id;
  String name;
  OfficePremises({ this.id, this.name });
  factory OfficePremises.fromJson(Map<String, dynamic> json) =>
      _$OfficePremisesFromJson(json);
  Map<String, dynamic> toJson() => _$OfficePremisesToJson(this);
}

OfficePremises _$OfficePremisesFromJson(Map<String, dynamic> json) {
  return OfficePremises(
    id: (json == null ? 0 : json[JsonFieldKey.OFFICE_PREMISES_ID] as int),
    name: (json == null ? "" : json[JsonFieldKey.OFFICE_PREMISES_NAME] as String)
  );
}

Map<String, dynamic> _$OfficePremisesToJson(OfficePremises instance) =>
    <String, dynamic> {
  JsonFieldKey.OFFICE_PREMISES_ID: instance.id,
  JsonFieldKey.OFFICE_PREMISES_NAME: instance.name
};

@JsonSerializable()
class Documents {
  Map<String, dynamic> data;
  Documents({ this.data });
  factory Documents.fromJson(Map<String, dynamic> json) => _$DocFromJson(json);
}

Documents _$DocFromJson(Map<String, dynamic> json) {
  return Documents(data: json );
}

@JsonSerializable()
class RemoveImage {
  String businessId;
  String imageUrl;
  String key;
  RemoveImage({this.businessId, this.imageUrl, this.key});
  Map<String, dynamic> toJson() => _$RemoveImageToJson(this);
}

Map<String, dynamic> _$RemoveImageToJson(RemoveImage instance) => <String, dynamic> {
  JsonFieldKey.BUSINESS_ID: instance.businessId,
  instance.key: instance.imageUrl
};

@JsonSerializable()
class Details {
  String doingBusinessName;
  String phoneNumber;
  String emailAddress;
  Details({ this.doingBusinessName, this.phoneNumber, this.emailAddress });
  factory Details.fromJson(Map<String, dynamic> json) => _$DetailsFromJson(json);
  Map<String, dynamic> toJson() => _$DetailsToJson(this);
}

Details _$DetailsFromJson(Map<String, dynamic> json) {
  return Details(
    doingBusinessName: json[JsonFieldKey.DOING_BUSINESS_AS] as String,
    phoneNumber: json[JsonFieldKey.MOBILE_NUMBER] as String,
    emailAddress: json[JsonFieldKey.EMAIL_ADDRESS] as String
  );
}

Map<String, dynamic> _$DetailsToJson(Details instance) => <String, dynamic> {
  JsonFieldKey.DOING_BUSINESS_AS: instance.doingBusinessName,
  JsonFieldKey.MOBILE_NUMBER: instance.phoneNumber,
  JsonFieldKey.EMAIL_ADDRESS: instance.emailAddress
};

@JsonSerializable()
class UserDetailsRequest {
  String businessId;
  UserDetailsRequest({ this.businessId });
  factory UserDetailsRequest.fromJson(Map<String, dynamic> json) =>
      _$UserDetailsRequestFromJson(json);
  Map<String, dynamic> toJson() => _$UserDetailsRequestToJson(this);
}

UserDetailsRequest _$UserDetailsRequestFromJson(Map<String, dynamic> json) {
  return UserDetailsRequest(
      businessId: json[JsonFieldKey.BUSINESS_ID] as String
  );
}

Map<String, dynamic> _$UserDetailsRequestToJson(UserDetailsRequest instance) =>
    <String, dynamic> {
      JsonFieldKey.BUSINESS_ID: instance.businessId
};